<?php
namespace Transact\FrontEnd\Controllers\AccountMeta;

use WP_User;

use Transact\Utils\Config\Parser\ConfigParser;
require_once  plugin_dir_path(__FILE__) . '../../utils/transact-utils-config-parser.php';

use Transact\FrontEnd\Controllers\Api\TransactApi;
require_once  plugin_dir_path(__FILE__) . 'transact-api.php';

/**
 * Class AccountMetaManager
 */
class AccountMetaManager
{
    const SUBSCRIPTION_INFO_META = 'xsact_subscription_info';
    const PURCHASE_INFO_META = 'xsact_purchase_info_';
    const XSACT_USER_ID_META = 'xsact_user_id';

    /**
     * @var WP_User|null
     */
    protected $cur_user;

    function __construct() {

        // safe to ignore input because validate_access is ONLY called
        // via transact-api  JS AJAX call.
        // phpcs:disable WordPressVIPMinimum.Variables.RestrictedVariables.cache_constraints___COOKIE

        $this->cur_user = wp_get_current_user();
    }

    /**
     * First check if the user is under any subscription plan (and is not expired). If yes, premium content will be shown.
     * If user is not subscribed, normal purchase check.
     *
     * @param $post_id
     * @return bool
     */
    function validate_access($post_id, $user_id = 0)
    {
        if ($user_id === 0) {
            $user_id = $this->cur_user->ID;
        }
        return ($this->validate_subscription($post_id, $user_id) || $this->validate_purchase($post_id, $user_id));
    }

    /**
     * Check if there is subscription on cookies and check its expiration date comparing with the account meta
     * @return bool
     */
    function validate_subscription($post_id, $user_id)
    {
        if ($this->cur_user) {
            $subscription_info = get_user_meta($user_id, self::SUBSCRIPTION_INFO_META, true);
            if ($subscription_info) {
                // if the subscription is expired, check to see if they have renewed on xsact
                if (time() * 1000 > $subscription_info['expires']) {
                    $valid = $this->refresh_subscription($user_id, $subscription_info, $post_id);
                    return $valid;
                }

                if (time() * 1000 - $subscription_info['validated'] < 60 * 60 * 24 * 1000) {
                    return true;
                } else {
                    // if the subscription needs to be re-validated, check to see if they still have a subscription on xsact
                    $valid = $this->refresh_subscription($user_id, $subscription_info, $post_id);
                    return $valid;
                }
            }
        }
        return false;
    }

    /**
     * Phone home to xsact and see if a subscription is valid. If it is, update the subscription validated meta
     * @return bool
     */
    function refresh_subscription($user_id, $subscription_info, $post_id) {
        $xsact_id = get_user_meta($user_id, self::XSACT_USER_ID_META, true);
        $options = get_option('transact-settings');
        $publisher_id = $options['account_id'];
        $subscription_id = $xsact_id . '_' . $publisher_id;
        // Check with api
        $validate_url = (new ConfigParser())->checkIfValidUserSubscription();
        $valid = (new TransactApi($post_id))->check_if_valid_user_subscription($validate_url, $publisher_id, $subscription_id);

        if($valid) {
            $update_subscription_info = array(
                'expires' => (gettype($valid) === 'boolean' ? $subscription_info['expires'] : $valid),
                'sale_id' => $subscription_info['sale_id'],
                'timestamp' => $subscription_info['timestamp'],
                'validated' => time() * 1000
            );
            update_user_meta(
                $user_id,
                self::SUBSCRIPTION_INFO_META,
                $update_subscription_info
            );
        } else {
            delete_user_meta(
                $user_id,
                self::SUBSCRIPTION_INFO_META
            );
        }

        return $valid;
    }

    /**
     * Check if there is a purchase of that post_id recorded on the user account
     * @param $post_id
     * @return bool
     */
    function validate_purchase($post_id, $user_id)
    {
        $meta_key = self::PURCHASE_INFO_META . $post_id;

        $purchase_info = get_user_meta($user_id, $meta_key, true);
        if ($purchase_info) {
            return true;
        }
        return false;
    }

    /**
     * Add record of a subscription on the user account
     * @param $post_id
     * @return bool
     */
    function add_subscription($expiration, $sales_id, $timestamp, $user_id = 0)
    {
        if(!$user_id) {
            $user_id = $this->cur_user->ID;
        }

        $new_subscription_info = array(
            'expires' => $expiration,
            'sale_id' => $sales_id,
            'timestamp' => $timestamp,
            'validated' => $timestamp
        );
        
        $subscription_info = get_user_meta($user_id, self::SUBSCRIPTION_INFO_META, true);
        if ($subscription_info) {
            update_user_meta(
                $user_id,
                self::SUBSCRIPTION_INFO_META,
                $new_subscription_info
            );
        } else {
            add_user_meta(
                $user_id,
                self::SUBSCRIPTION_INFO_META,
                $new_subscription_info
            );
        }
    }

    /**
     * Add record of a purchase on the user account
     * @param $post_id
     * @return bool
     */
    function add_purchase($post_id, $sales_id, $timestamp, $user_id = 0)
    {
        if(!$user_id) {
            $user_id = $this->cur_user->ID;
        }

        $new_purchase_info = array(
            'sale_id' => $sales_id,
            'timestamp' => $timestamp
        );
        $meta_key = self::PURCHASE_INFO_META . $post_id;

        $purchase_info = get_user_meta($user_id, $meta_key, true);
        if ($purchase_info) {
            update_user_meta(
                $user_id,
                $meta_key,
                $new_purchase_info
            );
        } else {
            add_user_meta(
                $user_id,
                $meta_key,
                $new_purchase_info
            );
        }
    }

    /**
     * Add metadata for tracking the transact account connected to this account
     * @param $post_id
     * @return bool
     */
    function connect_xsact_id_meta($wp_userid, $xsact_id) {
        add_user_meta(
            $wp_userid,
            self::XSACT_USER_ID_META,
            intval($xsact_id)
        );
    }

}
